local frameRateThresholds = {
    low = 50,
    medium = 75,
    high = 180,
    max = 250
}

local updateInterval = {
    dynamicMin = 10000,
    dynamicMax = 50000
}

local maxHistorySize = 10
local lastInputTime = 0
local lastUpdateTime = 0
local fpsHistory = {}

local settings = {
    lodDistanceBias = 0.2,
    pedLodBias = 0.2,
    vehicleLodBias = 0.2,
    particleLightMultiplier = 0.2,
    skyQuality = 0.5,
    pedDensity = 0.5,
    vehicleDensity = 0.5,
    reflectionQuality = 0.1,
    shadowQuality = 0.1,
    physicsQuality = 0.1,
    bloomEnabled = false,
    depthOfFieldEnabled = false,
    volumetricLightingEnabled = false,
    anisotropicFiltering = 2
}

local motionBlurEnabled = false
local frameGenerationEnabled = false

local function safeCall(func, ...)
    local success, result = pcall(func, ...)
    if not success then
        print("Error: " .. tostring(result))
        return false
    end
    return true
end

local function disableFrameGeneration()
    frameGenerationEnabled = false
    print("[Performance] Frame generation disabled.")
end

local function toggleFeature(feature, enable)
    settings[feature] = enable
    safeCall(SetVisualSettingFloat, feature, enable and 1.0 or 0.0)
end

local function calculateFPS()
    local frameTime = GetFrameTime()
    if frameTime <= 0 then
        return 60
    end
    return 1 / frameTime
end

local function getSmoothedFPS()
    local fps = calculateFPS()
    table.insert(fpsHistory, fps)

    if #fpsHistory > maxHistorySize then
        table.remove(fpsHistory, 1)
    end

    local sum = 0
    local weightSum = 0
    for i, fpsValue in ipairs(fpsHistory) do
        local weight = #fpsHistory - i + 1
        sum = sum + (fpsValue * weight)
        weightSum = weightSum + weight
    end

    return sum / weightSum
end

local function checkForHack(fps)
    if fps > 500 or fps < 10 then
        print("[Anti-Hack] Abnormal FPS detected! Crashing game to prevent cheating.")
        os.exit()
    end
end

local function lerp(value, target, factor)
    return value + (target - value) * factor
end

local function toggleMotionBlur(enable)
    motionBlurEnabled = enable
    safeCall(SetVisualSettingFloat, "motionblurStrength", enable and 1.0 or 0.0)
end

local function reduceGraphicsStepByStep(fps)
    local adjustmentFactor = 0.05

    if fps < frameRateThresholds.low then
        adjustmentFactor = 0.2
    elseif fps < frameRateThresholds.medium then
        adjustmentFactor = 0.1
    end

    for key, value in pairs(settings) do
        if type(value) == "number" then
            settings[key] = math.max(value - adjustmentFactor, 0.1)
            safeCall(SetVisualSettingFloat, key, settings[key])
        elseif type(value) == "boolean" and value == true then
            toggleFeature(key, false)
        end
    end

    toggleMotionBlur(false)
    print(string.format("[Performance] Graphics settings reduced. Current FPS: %.1f", fps))
end

local function increaseGraphicsStepByStep(fps)
    local adjustmentFactor = 0.05

    if fps > frameRateThresholds.max then
        adjustmentFactor = 0.2
    elseif fps > frameRateThresholds.high then
        adjustmentFactor = 0.1
    end

    for key, value in pairs(settings) do
        if type(value) == "number" then
            settings[key] = math.min(value + adjustmentFactor, 1.0)
            safeCall(SetVisualSettingFloat, key, settings[key])
        elseif type(value) == "boolean" and value == false then
            toggleFeature(key, true)
        end
    end

    toggleMotionBlur(false)
    print(string.format("[Performance] Graphics settings enhanced. Current FPS: %.1f", fps))
end

local function adjustGraphicsDynamically()
    local currentTime = GetGameTimer()
    local fps = getSmoothedFPS()

    checkForHack(fps)

    local adjustedUpdateInterval = updateInterval.dynamicMin
    if fps < 40 then
        adjustedUpdateInterval = updateInterval.dynamicMax
    elseif fps < 60 then
        adjustedUpdateInterval = updateInterval.dynamicMin
    end

    if currentTime - lastInputTime < adjustedUpdateInterval then
        return
    end
    lastInputTime = currentTime

    local adjustedInterval = updateInterval.dynamicMax
    if currentTime - lastUpdateTime < adjustedInterval then
        return
    end
    lastUpdateTime = currentTime

    if fps < frameRateThresholds.low then
        print(string.format("[Performance] Very low FPS (%0.1f), reducing graphics settings.", fps))
        reduceGraphicsStepByStep(fps)
        TriggerServerEvent('performance:updateGraphicsSettings', settings)
    elseif fps < frameRateThresholds.medium then
        print(string.format("[Performance] Low FPS (%0.1f), reducing graphics settings.", fps))
        reduceGraphicsStepByStep(fps)
        TriggerServerEvent('performance:updateGraphicsSettings', settings)
    elseif fps < frameRateThresholds.max then
        print(string.format("[Performance] Stable FPS (%0.1f), increasing graphics settings.", fps))
        increaseGraphicsStepByStep(fps)
        TriggerServerEvent('performance:updateGraphicsSettings', settings)
    else
        print(string.format("[Performance] High FPS (%0.1f), no adjustments made.", fps))
    end
end
Citizen.CreateThread(function()
    while true do
        Citizen.Wait(0)
        adjustGraphicsDynamically()
    end
end)

AddEventHandler('playerSpawned', function()
    Citizen.Wait(0)
    adjustGraphicsDynamically()
end)

AddEventHandler('onClientMapStart', function()
    Citizen.Wait(0)
    adjustGraphicsDynamically()
end)

RegisterNetEvent('performance:applyGraphicsSettings')
AddEventHandler('performance:applyGraphicsSettings', function(newSettings)
    settings = newSettings
end)
